import json
import logging

from .database.db import QolsysDB
from .enum import (
    PartitionAlarmState,
    PartitionAlarmType,
    PartitionSystemStatus,
)
from .observable import QolsysObservable
from .partition import QolsysPartition
from .settings import QolsysSettings
from .state import QolsysState
from .zone import QolsysZone
from .zwave_device import QolsysZWaveDevice
from .zwave_dimmer import QolsysDimmer
from .zwave_generic import QolsysGeneric
from .zwave_lock import QolsysLock
from .zwave_thermostat import QolsysThermostat

LOGGER = logging.getLogger(__name__)


class QolsysPanel(QolsysObservable):
    def __init__(self, settings: QolsysSettings, state: QolsysState) -> None:

        self._state = state
        self._settings = settings
        self._db = QolsysDB()

        # Partition settings
        self.settings_partition = ["SYSTEM_STATUS", "EXIT_SOUNDS", "ENTRY_DELAYS", "SYSTEM_STATUS_CHANGED_TIME"]
        self.state_partition = ["ALARM_STATE"]

        # Panel settings
        self.settings_panel_observer = QolsysObservable()
        self.settings_panel = [
            "PANEL_TAMPER_STATE", "AC_STATUS", "BATTERY_STATUS", "FAIL_TO_COMMUNICATE", "SECURE_ARMING", "AUTO_BYPASS",
            "AUTO_STAY", "AUTO_ARM_STAY", "AUTO_EXIT_EXTENSION", "FINAL_EXIT_DOOR_ARMING", "NO_ARM_LOW_BATTERY",
            "TEMPFORMAT", "LANGUAGE", "COUNTRY", "SYSTEM_TIME", "GSM_CONNECTION_STATUS", "GSM_SIGNAL_STRENGTH",
            "ANDROID_VERSION", "HARDWARE_VERSION", "TIMER_NORMAL_ENTRY_DELAY", "TIMER_NORMAL_EXIT_DELAY",
            "TIMER_LONG_ENTRY_DELAY", "TIMER_LONG_EXIT_DELAY", "ZWAVE_CONTROLLER", "ZWAVE_CARD", "POLICE_PANIC_ENABLED",
            "FIRE_PANIC_ENABLED", "AUXILIARY_PANIC_ENABLED", "NIGHTMODE_SETTINGS", "NIGHT_SETTINGS_STATE", "PARTITIONS",
            "SIX_DIGIT_USER_CODE", "SHOW_SECURITY_SENSORS", "SYSTEM_LOGGED_IN_USER", "PANEL_SCENES_SETTING", "CONTROL_4",
            "ZWAVE_FIRM_WARE_VERSION", "FINAL_EXIT_DOOR_ARMING", "NO_ARM_LOW_BATTERY", "MAC_ADDRESS",
            "LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N",
        ]

        self._PANEL_TAMPER_STATE = ""
        self._AC_STATUS = ""
        self._BATTERY_STATUS = ""
        self._FAIL_TO_COMMUNICATE = ""
        self._SECURE_ARMING = ""
        self._AUTO_BYPASS = ""
        self._AUTO_STAY = ""
        self._AUTO_ARM_STAY = ""
        self._AUTO_EXIT_EXTENSION = ""
        self._FINAL_EXIT_DOOR_ARMING = ""
        self._NO_ARM_LOW_BATTERY = ""
        self._TEMPFORMAT = ""
        self._LANGUAGE = ""
        self._COUNTRY = ""
        self._SYSTEM_TIME = ""
        self._GSM_CONNECTION_STATUS = ""
        self._GSM_SIGNAL_STRENGTH = ""
        self._ANDROID_VERSION = ""
        self._HARDWARE_VERSION = ""
        self._TIMER_NORMAL_ENTRY_DELAY = ""
        self._TIMER_NORMAL_EXIT_DELAY = ""
        self._TIMER_LONG_ENTRY_DELAY = ""
        self._TIMER_LONG_EXIT_DELAY = ""
        self._ZWAVE_FIRM_WARE_VERSION = ""
        self._ZWAVE_CONTROLLER = ""
        self._ZWAVE_CARD = ""
        self._POLICE_PANIC_ENABLED = ""
        self._FIRE_PANIC_ENABLED = ""
        self._AUXILIARY_PANIC_ENABfLED = ""
        self._NIGHTMODE_SETTINGS = ""
        self._NIGHT_SETTINGS_STATE = ""
        self._PARTITIONS = ""
        self._SIX_DIGIT_USER_CODE = ""
        self._SHOW_SECURITY_SENSORS = ""
        self._CONTROL_4 = ""
        self._MAC_ADDRESS = ""
        self._SYSTEM_LOGGED_IN_USER = ""
        self._PANEL_SCENES_SETTING = ""
        self._LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N = ""

        self._users = []
        self._unique_id = ""

        self._imei = ""
        self._product_type = ""

    def read_users_file(self) -> bool:
        # Loading user_code data from users.conf file if exists
        if self._settings.users_file_path.is_file():
            try:
                path = self._settings.users_file_path
                with path.open("r", encoding="utf-8") as file:
                    try:
                        users = json.load(file)
                        for user in users:
                            self._users.append(user)

                    except json.JSONDecodeError:
                        LOGGER.exception("users.conf file json error")
                        return False

            except FileNotFoundError:
                LOGGER.exception("users.conf file not found")
                return False

        return True

    @property
    def db(self) -> QolsysDB:
        return self._db

    @property
    def ANDROID_VERSION(self) -> str:
        self._ANDROID_VERSION = self.db.get_setting_panel("ANDROID_VERSION")
        return self._ANDROID_VERSION

    @property
    def PANEL_TAMPER_STATE(self) -> str:
        self._PANEL_TAMPER_STATE = self.db.get_setting_panel("PANEL_TAMPER_STATE")
        return self._PANEL_TAMPER_STATE

    @property
    def AC_STATUS(self) -> str:
        self._AC_STATUS = self.db.get_setting_panel("AC_STATUS")
        return self._AC_STATUS

    @property
    def BATTERY_STATUS(self) -> str:
        self._BATTERY_STATUS = self.db.get_setting_panel("BATTERY_STATUS")
        return self._BATTERY_STATUS

    @property
    def FAIL_TO_COMMUNICATE(self) -> str:
        self._FAIL_TO_COMMUNICATE = self.db.get_setting_panel("FAIL_TO_COMMUNICATE")
        return self._FAIL_TO_COMMUNICATE

    @property
    def SECURE_ARMING(self) -> str:
        self._SECURE_ARMING = self.db.get_setting_panel("SECURE_ARMING")
        return self._SECURE_ARMING

    @property
    def AUTO_BYPASS(self) -> str:
        self._AUTO_BYPASS = self.db.get_setting_panel("AUTO_BYPASS")
        return self._AUTO_BYPASS

    @property
    def AUTO_STAY(self) -> str:
        self._AUTO_STAY = self.db.get_setting_panel("AUTO_STAY")
        return self._AUTO_STAY

    @property
    def AUTO_ARM_STAY(self) -> str:
        self._AUTO_ARM_STAY = self.db.get_setting_panel("AUTO_ARM_STAY")
        return self._AUTO_ARM_STAY

    @property
    def AUTO_EXIT_EXTENSION(self) -> str:
        self._AUTO_EXIT_EXTENSION = self.db.get_setting_panel("AUTO_EXIT_EXTENSION")
        return self._AUTO_EXIT_EXTENSION

    @property
    def FINAL_EXIT_DOOR_ARMING(self) -> str:
        self._FINAL_EXIT_DOOR_ARMING = self.db.get_setting_panel("FINAL_EXIT_DOOR_ARMING")
        return self._FINAL_EXIT_DOOR_ARMING

    @property
    def NO_ARM_LOW_BATTERY(self) -> str:
        self._NO_ARM_LOW_BATTERY = self.db.get_setting_panel("NO_ARM_LOW_BATTERY")
        return self._NO_ARM_LOW_BATTERY

    @property
    def TEMPFORMAT(self) -> str:
        self._TEMPFORMAT = self.db.get_setting_panel("TEMPFORMAT")
        return self._TEMPFORMAT

    @property
    def LANGUAGE(self) -> str:
        self._LANGUAGE = self.db.get_setting_panel("LANGUAGE")
        return self._LANGUAGE

    @property
    def COUNTRY(self) -> str:
        self._COUNTRY = self.db.get_setting_panel("COUNTRY")
        return self._COUNTRY

    @property
    def SYSTEM_TIME(self) -> str:
        self._SYSTEM_TIME = self.db.get_setting_panel("SYSTEM_TIME")
        return self._SYSTEM_TIME

    @property
    def GSM_CONNECTION_STATUS(self) -> str:
        self._GSM_CONNECTION_STATUS = self.db.get_setting_panel("GSM_CONNECTION_STATUS")
        return self._GSM_CONNECTION_STATUS

    @property
    def GSM_SIGNAL_STRENGTH(self) -> str:
        self._GSM_SIGNAL_STRENGTH = self.db.get_setting_panel("GSM_SIGNAL_STRENGTH")
        return self._GSM_SIGNAL_STRENGTH

    @property
    def HARDWARE_VERSION(self) -> str:
        self._HARDWARE_VERSION = self.db.get_setting_panel("HARDWARE_VERSION")
        return self._HARDWARE_VERSION

    @property
    def ZWAVE_FIRM_WARE_VERSION(self) -> str:
        self._ZWAVE_FIRM_WARE_VERSION = self.db.get_setting_panel("ZWAVE_FIRM_WARE_VERSION")
        return self._ZWAVE_FIRM_WARE_VERSION

    @property
    def ZWAVE_CONTROLLER(self) -> str:
        self._ZWAVE_CONTROLLER = self.db.get_setting_panel("ZWAVE_CONTROLLER")
        return self._ZWAVE_CONTROLLER

    @property
    def ZWAVE_CARD(self) -> str:
        self._ZWAVE_CARD = self.db.get_setting_panel("ZWAVE_CARD")
        return self._ZWAVE_CARD

    @property
    def POLICE_PANIC_ENABLED(self) -> str:
        self._POLICE_PANIC_ENABLED = self.db.get_setting_panel("POLICE_PANIC_ENABLED")
        return self._POLICE_PANIC_ENABLED

    @property
    def FIRE_PANIC_ENABLED(self) -> str:
        self._FIRE_PANIC_ENABLED = self.db.get_setting_panel("FIRE_PANIC_ENABLED")
        return self._FIRE_PANIC_ENABLED

    @property
    def AUXILIARY_PANIC_ENABLED(self) -> str:
        self._AUXILIARY_PANIC_ENABLED = self.db.get_setting_panel("AUXILIARY_PANIC_ENABLED")
        return self._AUXILIARY_PANIC_ENABLED

    @property
    def PARTITIONS(self) -> str:
        self._PARTITIONS = self.db.get_setting_panel("PARTITIONS")
        return self._PARTITIONS

    @property
    def SIX_DIGIT_USER_CODE(self) -> str:
        self._SIX_DIGIT_USER_CODE = self.db.get_setting_panel("SIX_DIGIT_USER_CODE")
        return self._SIX_DIGIT_USER_CODE

    @property
    def CONTROL_4(self) -> str:
        self._CONTROL_4 = self.db.get_setting_panel("CONTROL_4")
        return self._CONTROL_4

    @property
    def NIGHTMODE_SETTINGS(self) -> str:
        self._NIGHTMODE_SETTINGS = self.db.get_setting_panel("NIGHTMODE_SETTINGS")
        return self._NIGHTMODE_SETTINGS

    @property
    def NIGHT_SETTINGS_STATE(self) -> str:
        self._NIGHT_SETTINGS_STATE = self.db.get_setting_panel("NIGHT_SETTINGS_STATE")
        return self._NIGHT_SETTINGS_STATE

    @property
    def SHOW_SECURITY_SENSORS(self) -> str:
        self._SHOW_SECURITY_SENSORS = self.db.get_setting_panel("SHOW_SECURITY_SENSORS")
        return self._SHOW_SECURITY_SENSORS

    @property
    def TIMER_NORMAL_ENTRY_DELAY(self) -> str:
        self._TIMER_NORMAL_ENTRY_DELAY = self.db.get_setting_panel("TIMER_NORMAL_ENTRY_DELAY")
        return self._TIMER_NORMAL_ENTRY_DELAY

    @property
    def TIMER_NORMAL_EXIT_DELAY(self) -> str:
        self._TIMER_NORMAL_EXIT_DELAY = self.db.get_setting_panel("TIMER_NORMAL_EXIT_DELAY")
        return self._TIMER_NORMAL_EXIT_DELAY

    @property
    def TIMER_LONG_ENTRY_DELAY(self) -> str:
        self._TIMER_LONG_ENTRY_DELAY = self.db.get_setting_panel("TIMER_LONG_ENTRY_DELAY")
        return self._TIMER_LONG_ENTRY_DELAY

    @property
    def MAC_ADDRESS(self) -> str:
        self._MAC_ADDRESS = self.db.get_setting_panel("MAC_ADDRESS")
        return self._MAC_ADDRESS

    @MAC_ADDRESS.setter
    def MAC_ADDRESS(self, value: str) -> None:
        self._MAC_ADDRESS = value

    @property
    def unique_id(self) -> str:
        mac_address = self.MAC_ADDRESS
        return mac_address.replace(":", "")

    @property
    def TIMER_LONG_EXIT_DELAY(self) -> str:
        self._TIMER_LONG_EXIT_DELAY = self.db.get_setting_panel("TIMER_LONG_ENTRY_DELAY")
        return self._TIMER_LONG_EXIT_DELAY

    @property
    def LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N(self) -> str:
        self._LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N = self.db.get_setting_panel("LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N")
        return self._LAST_UPDATE_IQ_REMOTE_PATCH_CKECKSUM_N

    @property
    def imei(self) -> str:
        return self._imei

    @imei.setter
    def imei(self, value: str) -> None:
        self._imei = value

    @property
    def product_type(self) -> str:
        return self._product_type

    @product_type.setter
    def product_type(self, value: str) -> None:
        self._product_type = value

    @property
    def SYSTEM_LOGGED_IN_USER(self) -> str:
        self._SYSTEM_LOGGED_IN_USER = self.db.get_setting_panel("SYSTEM_LOGGED_IN_USER")
        return self._SYSTEM_LOGGED_IN_USER

    @SYSTEM_LOGGED_IN_USER.setter
    def SYSTEM_LOGGED_IN_USER(self, value: str) -> None:
        self._SYSTEM_LOGGED_IN_USER = value

    @property
    def PANEL_SCENES_SETTING(self) -> str:
        self._PANEL_SCENES_SETTING = self.db.get_setting_panel("PANEL_SCENES_SETTING")
        return self.PANEL_SCENES_SETTING

    def load_database(self, database: dict) -> None:
        self.db.load_db(database)
        self._state.sync_partitions_data(self.get_partitions_from_db())
        self._state.sync_zones_data(self.get_zones_from_db())
        self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

    # Parse panel update to database
    def parse_iq2meid_message(self, data: dict) -> bool:  # noqa: C901, PLR0912, PLR0915

        eventName = data.get("eventName")
        dbOperation = data.get("dbOperation")
        uri = data.get("uri")

        match eventName:

            case "stopScreenCapture":
                pass

            case "dbChanged":

                match dbOperation:

                    case "update":
                        content_values = data.get("contentValues")
                        selection = data.get("selection")
                        selection_argument = data.get("selectionArgs")

                        match uri:

                            # Update Settings Content Provider
                            case self.db.table_qolsyssettings.uri:
                                name = content_values.get("name", "")
                                new_value = content_values.get("value", "")
                                old_value = self.db.get_setting_panel(name)
                                self.db.table_qolsyssettings.update(selection, selection_argument, content_values)

                                # Update Panel Settings - Send notification if settings ha changed
                                if name in self.settings_panel and old_value != new_value:
                                    LOGGER.debug("Panel Setting - %s: %s", name, new_value)
                                    self.settings_panel_observer.notify()

                                # Update Partition setting - Send notification if setting has changed
                                if name in self.settings_partition:
                                    partition_id = content_values.get("partition_id", "")
                                    partition = self._state.partition(partition_id)
                                    if partition is not None:
                                        match name:
                                            case "SYSTEM_STATUS":
                                                partition.system_status = PartitionSystemStatus(new_value)
                                            case "SYSTEM_STATUS_CHANGED_TIME":
                                                partition.system_status_changed_time = new_value
                                            case "EXIT_SOUNDS":
                                                partition.exit_sounds = new_value
                                            case "ENTRY_DELAYS":
                                                partition.entry_delays = new_value

                            # Update Sensor Content Provider
                            case self.db.table_sensor.uri:
                                self.db.table_sensor.update(selection, selection_argument, content_values)
                                zoneid = content_values.get("zoneid", "")
                                zone = self._state.zone(zoneid)
                                if zone is not None:
                                    zone.update(content_values)

                            # Update State
                            case self.db.table_state.uri:
                                name = content_values.get("name", "")
                                new_value = content_values.get("value", "")
                                partition_id = content_values.get("partition_id", "")
                                self.db.table_state.update(selection, selection_argument, content_values)

                                if name in self.state_partition:
                                    partition = self._state.partition(partition_id)
                                    if partition is not None:
                                        match name:
                                            case "ALARM_STATE":
                                                partition.alarm_state = PartitionAlarmState(new_value)

                            # Update heat_map
                            case self.db.table_heat_map.uri:
                                self.db.table_heat_map.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update master_slave
                            case self.db.table_master_slave.uri:
                                self.db.table_master_slave.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update dashboard_msgs
                            case self.db.table_dashboard_msgs.uri:
                                self.db.table_dashboard_msgs.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update PartitionContentProvider
                            case self.db.table_partition.uri:
                                self.db.table_partition.update(selection, selection_argument, content_values)
                                partition_id = content_values.get("partition_id", "")
                                partition = self._state.partition(partition_id)
                                if partition is not None:
                                    partition.update_partition(content_values)

                            # Update History Content Provider
                            case self.db.table_history.uri:
                                self.db.table_history.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update DimmerLightsContentProvider
                            case self.db.table_dimmer.uri:
                                self.db.table_dimmer.update(selection, selection_argument, content_values)
                                node_id = content_values.get("node_id", "")
                                node = self._state.zwave_device(node_id)
                                if node is not None and isinstance(node, QolsysDimmer):
                                    node.update_dimmer(content_values)

                            # Update Thermostat Content Provider
                            case self.db.table_thermostat.uri:
                                self.db.table_thermostat.update(selection, selection_argument, content_values)
                                node_id = content_values.get("node_id", "")
                                node = self._state.zwave_device(node_id)
                                if node is not None and isinstance(node, QolsysThermostat):
                                    node.update_thermostat(content_values)

                            # Update DoorLockContentProvider
                            case self.db.table_doorlock.uri:
                                self.db.table_doorlock.update(selection, selection_argument, content_values)
                                node_id = content_values.get("node_id", "")
                                node = self._state.zwave_device(node_id)
                                if node is not None and isinstance(node, QolsysLock):
                                    node.update_lock(content_values)

                            # Update ZwaveContentProvider
                            case self.db.table_zwave_node.uri:
                                self.db.table_zwave_node.update(selection, selection_argument, content_values)
                                node_id = content_values.get("node_id", "")
                                node = self._state.zwave_device(node_id)
                                if node is not None:
                                    node.update_base(content_values)

                            # Update Z-Wave History Content Provier
                            case self.db.table_zwave_history.uri:
                                self.db.table_zwave_history.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update AutomationDeviceContentProvider
                            case self.db.table_automation.uri:
                                self.db.table_automation.update(selection, selection_argument, content_values)
                                # No action needed

                            # Update Alarmed Sensor Content Provider
                            case self.db.table_alarmedsensor.uri:
                                self.db.table_alarmedsensor.update(selection, selection_argument, content_values)

                            # Update IQ Remote Settings Content Provider
                            case self.db.table_iqremotesettings.uri:
                                self.db.table_iqremotesettings.update(selection, selection_argument, content_values)
                                # No action needed

                            case _:
                                LOGGER.debug("iq2meid updating unknow uri:%s", uri)
                                LOGGER.debug(data)

                    case "delete":
                        selection = data.get("selection")
                        selection_argument = data.get("selectionArgs")

                        match uri:

                            case self.db.table_sensor.uri:
                                self.db.table_sensor.delete(selection, selection_argument)
                                self._state.sync_zones_data(self.get_zones_from_db())
                                # Notify delete zone

                            case self.db.table_iqremotesettings.uri:
                                self.db.table_iqremotesettings.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_state.uri:
                                self.db.table_state.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_master_slave.uri:
                                self.db.table_master_slave.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_qolsyssettings.uri:
                                self.db.table_qolsyssettings.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_alarmedsensor.uri:
                                self.db.table_alarmedsensor.delete(selection, selection_argument)
                                self._state.sync_partitions_data(self.get_partitions_from_db())

                            case self.db.table_history.uri:
                                self.db.table_history.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_zwave_history.uri:
                                self.db.table_zwave_history.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_doorlock.uri:
                                self.db.table_doorlock.delete(selection, selection_argument)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            case self.db.table_dimmer.uri:
                                self.db.table_dimmer.delete(selection, selection_argument)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            case self.db.table_thermostat.uri:
                                self.db.table_thermostat.delete(selection, selection_argument)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            case self.db.table_zwave_node.uri:
                                self.db.table_zwave_node.delete(selection, selection_argument)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            case self.db.table_automation.uri:
                                self.db.table_automation.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_partition.uri:
                                self.db.table_partition.delete(selection, selection_argument)
                                self._state.sync_partitions_data(self.get_partitions_from_db())

                            case self.db.table_user.uri:
                                self.db.table_user.delete(selection, selection_argument)
                                # No action needed

                            case self.db.table_dashboard_msgs.uri:
                                self.db.table_dashboard_msgs.delete(selection, selection_argument)
                                # No action needed

                            case _:
                                LOGGER.debug("iq2meid deleting unknown uri:%s", uri)
                                LOGGER.debug(data)

                    case "insert":
                        content_values = data.get("contentValues")

                        match uri:

                            # Inser State Content Provider
                            case self.db.table_state.uri:
                                self.db.table_state.insert(data=content_values)

                                name = content_values.get("name", "")
                                new_value = content_values.get("value", "")
                                if name in self.state_partition:
                                    partition_id = content_values.get("partition_id", "")
                                    partition = self._state.partition(partition_id)
                                    if partition is not None:
                                        match name:
                                            case "ALARM_STATE":
                                                partition.alarm_state = PartitionAlarmState(new_value)

                            # Inser Partition Content Provider
                            case self.db.table_partition:
                                self.db.table_partition.insert(data=content_values)
                                self._state.sync_partitions_data(self.get_partitions_from_db())

                            # Insert Settings Content Provider
                            case self.db.table_qolsyssettings.uri:
                                self.db.table_qolsyssettings.insert(data=content_values)

                                # Update Partition setting - Send notification if setting has changed
                                name = content_values.get("name", "")
                                new_value = content_values.get("value", "")
                                if name in self.settings_partition:
                                    partition_id = content_values.get("partition_id", "")
                                    partition = self._state.partition(partition_id)
                                    if partition is not None:
                                        match name:
                                            case "SYSTEM_STATUS":
                                                partition.system_status = PartitionSystemStatus(new_value)
                                            case "SYSTEM_STATUS_CHANGED_TIME":
                                                partition.system_status_changed_time = new_value
                                            case "EXIT_SOUNDS":
                                                partition.exit_sounds = new_value
                                            case "ENTRY_DELAYS":
                                                partition.entry_delays = new_value

                            # UserContentProvider
                            case self.db.table_user.uri:
                                self.db.table_user.insert(data=content_values)
                                # No action needed

                            # MasterSlave Content Provider
                            case self.db.table_master_slave.uri:
                                self.db.table_master_slave.insert(data=content_values)
                                # No action needed

                            # Automation Content Provider
                            case self.db.table_automation.uri:
                                self.db.table_automation.insert(content_values)
                                # No action needed

                            # Sensor Content Provider
                            case self.db.table_sensor.uri:
                                self.db.table_sensor.insert(data=content_values)
                                self._state.sync_zones_data(self.get_zones_from_db())

                            # Door Lock Content Provider
                            case self.db.table_doorlock.uri:
                                self.db.table_doorlock.insert(data=content_values)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            # Dimmer Content Provider
                            case self.db.table_dimmer.uri:
                                self.db.table_dimmer.insert(data=content_values)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            # Thermostat Content Provider
                            case self.db.table_thermostat.uri:
                                self.db.table_thermostat.insert(data=content_values)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            # ZWave Node Content Provider
                            case self.db.table_zwave_node.uri:
                                self.db.table_zwave_node.insert(data=content_values)
                                self._state.sync_zwave_devices_data(self.get_zwave_devices_from_db())

                            # HistoryContentProvider
                            case self.db.table_history.uri:
                                self.db.table_history.insert(data=content_values)
                                # No action needed

                            # AlarmedSensorProvider
                            case self.db.table_alarmedsensor.uri:

                                partition_id = content_values.get("partition_id", "")
                                self.db.table_alarmedsensor.insert(data=content_values)

                                partition = self._state.partition(partition_id)
                                if partition is not None:
                                    partition.append_alarm_type([PartitionAlarmType(content_values.get("sgroup", ""))])

                            # IQRemoteSettingsProvider
                            case self.db.table_iqremotesettings.uri:
                                self.db.table_iqremotesettings.insert(data=content_values)
                                # No action needed

                            # HeatMapContentProvider
                            case self.db.table_heat_map.uri:
                                self.db.table_heat_map.insert(data=content_values)
                                # No action needed

                            # ZDeviceHistoryContentProvider
                            case self.db.table_zwave_history.uri:
                                self.db.table_zwave_history.insert(data=content_values)
                                # No action needed

                            # Dashboard Message Content Provider
                            case self.db.table_dashboard_msgs.uri:
                                self.db.table_dashboard_msgs.insert(data=content_values)
                                # No action needed

                            case _:
                                LOGGER.debug("iq2meid inserting unknow uri:%s", uri)
                                LOGGER.debug(data)

                    case _:
                        LOGGER.debug("iq2meid - Unknow dboperation: %s", dbOperation)
                        LOGGER.debug(data)
            case _:
                LOGGER.debug("iq2meid - Unknow event: %s", eventName)
                LOGGER.debug(data)

    def check_user(self, user_code: str) -> int:
        for user in self._users:
            if user["user_code"] == user_code:
                return user["id"]

        # No valid user code found
        return -1

    def get_zwave_devices_from_db(self) -> list[QolsysZWaveDevice]:
        devices = []
        devices_list = self.db.get_zwave_devices()
        dimmers_list = self.db.get_dimmers()
        thermostats_list = self.db.get_thermostats()
        locks_list = self.db.get_locks()

        for device in devices_list:

            device_added = False

            zwave_node_id = device.get("node_id", "")
            # Check if z-wave device is a Dimmer
            for d in dimmers_list:
                dimmer_node_id = d.get("node_id", "")

                # Found a Dimmer
                if zwave_node_id == dimmer_node_id:
                    qolsys_dimmer = QolsysDimmer(d, device)
                    devices.append(qolsys_dimmer)
                    device_added = True
                    break

            # Check is z-wave devie is a Thermostat
            for thermostat in thermostats_list:
                thermostat_node_id = thermostat.get("node_id", "")

                # Found a Thermostat
                if zwave_node_id == thermostat_node_id:
                    qolsys_thermostat = QolsysThermostat(thermostat, device)
                    devices.append(qolsys_thermostat)
                    device_added = True
                    break

            # Check is z-wave device is a Lock
            for lock in locks_list:
                lock_node_id = lock.get("node_id", "")

                # Found a Lock
                if zwave_node_id == lock_node_id:
                    qolsys_lock = QolsysLock(lock, device)
                    devices.append(qolsys_lock)
                    device_added = True
                    break

            # Found a SmartOutlet

            # Found Garage Door Openner

            # No Specific z-wave device found, add a generic z-wave device
            if not device_added:
                qolsys_generic = QolsysGeneric()
                devices.append(qolsys_generic)

        return devices

    def get_zones_from_db(self) -> list[QolsysZone]:
        zones = []
        zones_list: list[dict] = self.db.get_zones()

        # Create sensors array
        for zone_info in zones_list:
            zones.append(QolsysZone(zone_info))

        return zones

    def get_partitions_from_db(self) -> list[QolsysPartition]:

        partitions = []
        partition_list: list[dict] = self.db.get_partitions()

        # Create partitions array
        for partition_dict in partition_list:

            partition_id = partition_dict["partition_id"]

            settings_dict = {
                "SYSTEM_STATUS": self.db.get_setting_partition("SYSTEM_STATUS", partition_id) or "UNKNOWN",
                "SYSTEM_STATUS_CHANGED_TIME": self.db.get_setting_partition("SYSTEM_STATUS_CHANGED_TIME", partition_id) or "",
                "EXIT_SOUNDS": self.db.get_setting_partition("EXIT_SOUNDS", partition_id) or "",
                "ENTRY_DELAYS": self.db.get_setting_partition("ENTRY_DELAYS", partition_id) or "",
            }

            alarm_type = []
            for alarm in self.db.get_alarm_type(partition_id):
                if alarm == "":
                    alarm_type.append(PartitionAlarmType("Police Emergency"))
                else:
                    alarm_type.append(PartitionAlarmType(alarm))

            alarm_state = PartitionAlarmState(self.db.get_state_partition("ALARM_STATE", partition_id) or "UNKNOWN")

            partition = QolsysPartition(partition_dict, settings_dict, alarm_state, alarm_type)
            partitions.append(partition)

        return partitions

    def dump(self) -> None:
        LOGGER.debug("*** Qolsys Panel Information ***")
        LOGGER.debug("Android Version: %s", self.ANDROID_VERSION)
        LOGGER.debug("Hardware Version: %s", self.HARDWARE_VERSION)
        LOGGER.debug("MAC Address: %s", self.MAC_ADDRESS)
        LOGGER.debug("Unique ID: %s", self.unique_id)
        LOGGER.debug("Panel Tamper State: %s", self.PANEL_TAMPER_STATE)
        LOGGER.debug("AC Status: %s", self.AC_STATUS)
        LOGGER.debug("Battery Status: %s", self.BATTERY_STATUS)
        LOGGER.debug("GSM Connection Status: %s", self.GSM_CONNECTION_STATUS)
        LOGGER.debug("GSM Signal Strength: %s", self.GSM_SIGNAL_STRENGTH)
        LOGGER.debug("Fail To Communicate: %s", self.FAIL_TO_COMMUNICATE)
        # LOGGER.debug("System Time: %s",datetime.fromtimestamp(int(self.SYSTEM_TIME)/1000))
        LOGGER.debug("Country: %s", self.COUNTRY)
        LOGGER.debug("Language: %s", self.LANGUAGE)
        LOGGER.debug("Temp Format: %s", self.TEMPFORMAT)
        LOGGER.debug("Z-Wave Firmware Version: %s", self.ZWAVE_FIRM_WARE_VERSION)
        LOGGER.debug("Z-Wave Card Present: %s", self.ZWAVE_CARD)
        LOGGER.debug("Z-Wave Controller Enabled: %s", self.ZWAVE_CONTROLLER)
        LOGGER.debug("Partitons Enabled: %s", self.PARTITIONS)
        LOGGER.debug("Control4 Enabled: %s", self.CONTROL_4)
        LOGGER.debug("Six Digit User Code Enabled: %s", self.SIX_DIGIT_USER_CODE)
        LOGGER.debug("Secure Arming: %s", self.SECURE_ARMING)
        LOGGER.debug("Auto-Stay: %s", self.AUTO_STAY)
        LOGGER.debug("Auto-Bypass: %s", self.AUTO_BYPASS)
        LOGGER.debug("Auto-Arm-Stay: %s", self.AUTO_ARM_STAY)
        LOGGER.debug("Auto-Exit-Extension: %s", self.AUTO_EXIT_EXTENSION)
        LOGGER.debug("Final-Exit-Door-Arming: %s", self.FINAL_EXIT_DOOR_ARMING)
        LOGGER.debug("No-Arm-Low-Battery: %s", self.NO_ARM_LOW_BATTERY)
        LOGGER.debug("Normal Entry Delay: %s", self.TIMER_NORMAL_ENTRY_DELAY)
        LOGGER.debug("Normal Exit Delay: %s", self.TIMER_NORMAL_EXIT_DELAY)
        LOGGER.debug("Long Entry Delay: %s", self.TIMER_LONG_ENTRY_DELAY)
        LOGGER.debug("Long Exit Delay: %s", self.TIMER_LONG_EXIT_DELAY)
        LOGGER.debug("Auxiliary Panic Enabled: %s", self.AUXILIARY_PANIC_ENABLED)
        LOGGER.debug("Fire Panic Enabled: %s", self.FIRE_PANIC_ENABLED)
        LOGGER.debug("Police Panic Enabled: %s", self.POLICE_PANIC_ENABLED)
        LOGGER.debug("Night Mode Settings: %s", self.NIGHTMODE_SETTINGS)
        LOGGER.debug("Night Mode Settings Stage: %s", self.NIGHT_SETTINGS_STATE)
        LOGGER.debug("Show Security Sensors: %s", self.SHOW_SECURITY_SENSORS)

        LOGGER.debug("Users list:")
        for user in self._users:
            LOGGER.debug("User: %s", user["id"])



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableAutomation(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.AutomationDeviceContentProvider/automation"
        self._table = "automation"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "virtual_node_id",
            "version",
            "opr",
            "partition_id",
            "end_point",
            "extras",
            "is_autolocking_enabled",
            "device_type",
            "endpoint_secure_cmd_classes",
            "automation_id",
            "device_name",
            "protocol",
            "node_battery_level_value",
            "state",
            "last_updated_date",
            "manufacturer_id",
            "endpoint_cmd_classes",
            "device_id",
            "nodeid_cmd_classes",
            "is_device_hidden",
            "nodeid_secure_cmd_classes",
            "created_date",
            "status",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,virtual_node_id,version,opr,partition_id,end_point,extras,
                             is_autolocking_enabled,device_type,endpoint_secure_cmd_classes,automation_id,device_name,
                             protocol,node_battery_level_value,state,last_updated_date,manufacturer_id,
                             endpoint_cmd_classes,device_id,nodeid_cmd_classes,is_device_hidden,nodeid_secure_cmd_classes,
                             created_date,status) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("virtual_node_id", ""),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("end_point", ""),
            data.get("extras", ""),
            data.get("is_autolocking_enabled", ""),
            data.get("device_type", ""),
            data.get("endpoint_secure_cmd_classes", ""),
            data.get("automation_id", ""),
            data.get("device_name", ""),
            data.get("protocol", ""),
            data.get("node_battery_level_value", ""),
            data.get("state", ""),
            data.get("last_updated_date", ""),
            data.get("manufacturer_id", ""),
            data.get("endpoint_cmd_classes", ""),
            data.get("device_id", ""),
            data.get("nodeid_cmd_classes", ""),
            data.get("is_device_hidden", ""),
            data.get("nodeid_secure_cmd_classes", ""),
            data.get("created_date", ""),
            data.get("status", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable
from .table_alarmedsensor import QolsysTableAlarmedSensor
from .table_automation import QolsysTableAutomation
from .table_country_locale import QolsysTableCountryLocale
from .table_dashboard_msgs import QolsysTableDashboardMsgs
from .table_dimmerlight import QolsysTableDimmerLight
from .table_doorlock import QolsysTableDoorLock
from .table_eu_event import QolsysTableEuEvent
from .table_heat_map import QolsysTableHeatMap
from .table_history import QolsysTableHistory
from .table_iqremotesettings import QolsysTableIqRemoteSettings
from .table_iqrouter_network_config import QolsysTableIqRouterNetworkConfig
from .table_iqrouter_user_device import QolsysTableIqRouterUserDevice
from .table_master_slave import QolsysTableMasterSlave
from .table_nest_device import QolsysTableNestDevice
from .table_output_rules import QolsysTableOutputRules
from .table_partition import QolsysTablePartition
from .table_pgm_outputs import QolsysTablePgmOutputs
from .table_powerg_device import QolsysTablePowerGDevice
from .table_qolsyssettings import QolsysTableQolsysSettings
from .table_scene import QolsysTableScene
from .table_sensor import QolsysTableSensor
from .table_shades import QolsysTableShades
from .table_smartsocket import QolsysTableSmartSocket
from .table_state import QolsysTableState
from .table_tcc import QolsysTableTcc
from .table_thermostat import QolsysTableThermostat
from .table_trouble_conditions import QolsysTableTroubleConditions
from .table_user import QolsysTableUser
from .table_virtual_device import QolsysTableVirtualDevice
from .table_weather import QolsysTableWeather
from .table_zigbee_device import QolsysTableZigbeeDevice
from .table_zwave_association_group import QolsysTableZwaveAssociationGroup
from .table_zwave_history import QolsysTableZwaveHistory
from .table_zwave_node import QolsysTableZwaveNode
from .table_zwave_other import QolsysTableZwaveOther

LOGGER = logging.getLogger(__name__)


class QolsysDB:

    def __init__(self) -> None:  # noqa: PLR0915

        self._db: sqlite3.Connection = sqlite3.connect(":memory:")
        self._cursor: sqlite3.Cursor = self._db.cursor()

        self.table_alarmedsensor = QolsysTableAlarmedSensor(self.db, self.cursor)
        self.table_automation = QolsysTableAutomation(self.db, self.cursor)
        self.table_country_locale = QolsysTableCountryLocale(self.db, self.cursor)
        self.table_dashboard_msgs = QolsysTableDashboardMsgs(self.db, self.cursor)
        self.table_dimmer = QolsysTableDimmerLight(self.db, self.cursor)
        self.table_doorlock = QolsysTableDoorLock(self.db, self.cursor)
        self.table_eu_event = QolsysTableEuEvent(self.db, self.cursor)
        self.table_heat_map = QolsysTableHeatMap(self.db, self.cursor)
        self.table_history = QolsysTableHistory(self.db, self.cursor)
        self.table_iqremotesettings = QolsysTableIqRemoteSettings(self.db, self.cursor)
        self.table_iqrouter_network_config = QolsysTableIqRouterNetworkConfig(self.db, self.cursor)
        self.table_iqrouter_user_device = QolsysTableIqRouterUserDevice(self.db, self.cursor)
        self.table_master_slave = QolsysTableMasterSlave(self.db, self.cursor)
        self.table_partition = QolsysTablePartition(self.db, self.cursor)
        self.table_powerg_device = QolsysTablePowerGDevice(self.db, self.cursor)
        self.table_sensor = QolsysTableSensor(self.db, self.cursor)
        self.table_smartsocket = QolsysTableSmartSocket(self.db, self.cursor)
        self.table_qolsyssettings = QolsysTableQolsysSettings(self.db, self.cursor)
        self.table_scene = QolsysTableScene(self.db, self.cursor)
        self.table_state = QolsysTableState(self.db, self.cursor)
        self.table_tcc = QolsysTableTcc(self.db, self.cursor)
        self.table_thermostat = QolsysTableThermostat(self.db, self.cursor)
        self.table_trouble_conditions = QolsysTableTroubleConditions(self.db, self.cursor)
        self.table_user = QolsysTableUser(self.db, self.cursor)
        self.table_virtual_device = QolsysTableVirtualDevice(self.db, self.cursor)
        self.table_weather = QolsysTableWeather(self.db, self.cursor)
        self.table_zigbee_device = QolsysTableZigbeeDevice(self.db, self.cursor)
        self.table_zwave_association_goup = QolsysTableZwaveAssociationGroup(self.db, self.cursor)
        self.table_zwave_history = QolsysTableZwaveHistory(self.db, self.cursor)
        self.table_zwave_node = QolsysTableZwaveNode(self.db, self.cursor)
        self.table_zwave_other = QolsysTableZwaveOther(self.db, self.cursor)
        self.table_pgm_outputs = QolsysTablePgmOutputs(self.db, self.cursor)
        self.table_output_rules = QolsysTableOutputRules(self.db, self.cursor)
        self.table_shades = QolsysTableShades(self.db, self.cursor)
        self.table_nest_device = QolsysTableNestDevice(self.db, self.cursor)

        self._table_array = []
        self._table_array.append(self.table_sensor)
        self._table_array.append(self.table_partition)
        self._table_array.append(self.table_qolsyssettings)
        self._table_array.append(self.table_alarmedsensor)
        self._table_array.append(self.table_state)
        self._table_array.append(self.table_history)
        self._table_array.append(self.table_zwave_node)
        self._table_array.append(self.table_zwave_history)
        self._table_array.append(self.table_iqremotesettings)
        self._table_array.append(self.table_iqrouter_network_config)
        self._table_array.append(self.table_iqrouter_user_device)
        self._table_array.append(self.table_thermostat)
        self._table_array.append(self.table_dimmer)
        self._table_array.append(self.table_doorlock)
        self._table_array.append(self.table_smartsocket)
        self._table_array.append(self.table_automation)
        self._table_array.append(self.table_dashboard_msgs)
        self._table_array.append(self.table_heat_map)
        self._table_array.append(self.table_master_slave)
        self._table_array.append(self.table_scene)
        self._table_array.append(self.table_user)
        self._table_array.append(self.table_country_locale)
        self._table_array.append(self.table_zwave_other)
        self._table_array.append(self.table_tcc)
        self._table_array.append(self.table_weather)
        self._table_array.append(self.table_powerg_device)
        self._table_array.append(self.table_trouble_conditions)
        self._table_array.append(self.table_zigbee_device)
        self._table_array.append(self.table_zwave_association_goup)
        self._table_array.append(self.table_virtual_device)
        self._table_array.append(self.table_eu_event)
        self._table_array.append(self.table_pgm_outputs)
        self._table_array.append(self.table_output_rules)
        self._table_array.append(self.table_shades)
        self._table_array.append(self.table_nest_device)

        # Other Table not Implemented
        # content://com.qolsys.qolsysprovider.AllSensorsContentProvider/all_sensor
        # content://com.qolsys.qolsysprovider.CameraRequestContentProvider/camerarequest
        # content://com.qolsys.qolsysprovider.FAQContentProvider/faq
        # content://com.qolsys.qolsysprovider.HomeAutomationEventsContentProvider/home_automation_event
        # content://com.qolsys.qolsysprovider.HomeAutomationPendingContentProvider/home_automation_pending
        # content://com.qolsys.qolsysprovider.HomeAutomationRulesContentProvider/home_automation_rule
        # content://com.qolsys.qolsysprovider.ZigbeeSmartEnergyContentProvider/zigbee_smart_energy
        # content://com.qolsys.qolsysprovider.ProvisionListContentProvider/provision_list
        # content://com.qolsys.qolsysprovider.PowerGSignalStrengthContentProvider/powerg_signal_strength
        # content://com.qolsys.qolsysprovider.ProximityTagContentProvider/proximity_tag
        # content://com.qolsys.qolsysprovider.OutputRulesContentProvider/output_rules
        # content://com.qolsys.qolsysprovider.IQCameraContentProvider/iqcamera
        # content://com.qolsys.qolsysprovider.CorbusDeviceContentProvider/corbus_device
        # content://com.qolsys.qolsysprovider.AutomationRulesContentProvider/automation_rules
        # content://com.qolsys.qolsysprovider.WifiClientsContentProvider/wifi_clients_table
        # content://com.qolsys.qolsysprovider.MxLoopControllerContentProvider/mx_loop_controller
        # content://com.qolsys.qolsysprovider.SystemCheckContentProvider/system_check
        # content://com.qolsys.qolsysprovider.BluetoothDevicesContentProvider/bluetooth_devices
        # content://com.qolsys.qolsysprovider.AxonDeviceContentProvider/axon_device
        # content://com.qolsys.qolsysprovider.AxonSignalStrengthContentProvider/axon_signal_strength
        # content://com.qolsys.qolsysprovider.PanelInfoContentProvider/panel_info
        # content://com.qolsys.qolsysprovider.AxonRSSIContentProvider/axon_rssi_table
        # content://com.qolsys.qolsysprovider.PowerGDeviceContentProvider/powerg_device
        # content://com.qolsys.qolsysprovider.PowerGRSSIContentProvider/powerg_rssi_table
        # content://com.qolsys.qolsysprovider.PgmOutputsContentProvider/pgm_outputs

    @property
    def db(self) -> sqlite3.Connection:
        return self._db

    @property
    def cursor(self) -> sqlite3.Cursor:
        return self._cursor

    def get_partitions(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_partition.table} ORDER BY partition_id")
        self.db.commit()

        partitions = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            partitions.append(row_dict)

        return partitions

    def get_zwave_devices(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_zwave_node.table} ORDER BY node_id")
        self.db.commit()

        devices = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            devices.append(row_dict)

        return devices

    def get_locks(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_doorlock.table} ORDER BY node_id")
        self.db.commit()

        locks = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            locks.append(row_dict)

        return locks

    def get_thermostats(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_thermostat.table} ORDER BY node_id")
        self.db.commit()

        thermostats = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            thermostats.append(row_dict)

        return thermostats

    def get_dimmers(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_dimmer.table} ORDER BY node_id")
        self.db.commit()

        dimmers = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            dimmers.append(row_dict)

        return dimmers

    def get_zones(self) -> list[dict]:
        self.cursor.execute(f"SELECT * FROM {self.table_sensor.table} ORDER BY zoneid")
        self.db.commit()

        zones = []
        columns = [description[0] for description in self.cursor.description]
        for row in self.cursor.fetchall():
            row_dict = dict(zip(columns, row, strict=True))
            zones.append(row_dict)

        return zones

    def get_setting_panel(self, setting: str) -> str:
        self.cursor.execute(f"""SELECT value FROM {self.table_qolsyssettings.table}
                             WHERE name = ? and partition_id  = ? """, (setting, "0"))
        row = self.cursor.fetchone()

        if row is None:
            LOGGER.debug("%s value not found", setting)
            return None

        return row[0]

    def get_setting_partition(self, setting: str, partition_id: str) -> str | None:
        self.cursor.execute(f"""SELECT value FROM {self.table_qolsyssettings.table}
                             WHERE name = ? and partition_id  = ? """, (setting, partition_id))
        row = self.cursor.fetchone()

        if row is None:
            LOGGER.debug("%s value not found", setting)
            return None

        return row[0]

    def get_state_partition(self, state: str, partition_id: str) -> str | None:
        self.cursor.execute(
            f"""SELECT value FROM {self.table_state.table} WHERE name = ? and partition_id  = ? """, (state, partition_id))
        row = self.cursor.fetchone()

        if row is None:
            LOGGER.debug("%s value not found", state)
            return None

        return row[0]

    def get_alarm_type(self, partition_id: str) -> list[str]:
        alarm_type = []
        self.cursor.execute(f"SELECT sgroup FROM {self.table_alarmedsensor.table} WHERE partition_id  = ? ", (partition_id))
        rows = self.cursor.fetchall()

        for row in rows:
            alarm_type.append(row[0])

        return alarm_type

    def clear_db(self) -> None:
        for table in self._table_array:
            table.clear()

    def get_table(self, uri: str) -> QolsysTable | None:
        for table in self._table_array:
            if uri == table.uri:
                return table

        return None

    def load_db(self, database: dict) -> None:

        self.clear_db()

        if not database:
            LOGGER.error("Loading Database Error, No Data Provided")
            return

        for uri in database:
            table = self.get_table(uri.get("uri"))

            if table is None:
                LOGGER.error("Please Report")
                LOGGER.error("Loading Unknown databse URI")
                LOGGER.error(uri)
                continue

            for u in uri.get("resultSet"):
                table.insert(data=u)



import asyncio


class QolsysMqttCommandQueue:
    def __init__(self) -> None:
        self.lock = asyncio.Lock()
        self.waiters: dict[str, asyncio.Future] = {}

    async def handle_response(self, response: dict) -> None:
        requestID = response.get("requestID")

        if not requestID:
            msg = "MQTT Command response must have a requestID"
            raise ValueError(msg)

        async with self.lock:
            future = self.waiters.pop(requestID, None)

        if future and not future.done():
            future.set_result(response)

    async def wait_for_response(self, request_id: str) -> None:
        if request_id in self.waiters:
            msg = f"Duplicate waiter for request_id: {request_id}"
            raise ValueError(msg)

        future = asyncio.get_event_loop().create_future()
        async with self.lock:
            self.waiters[request_id] = future

        return await future



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableCountryLocale(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.CountryLocaleContentProvider/country_locale"
        self._table = "country_locale"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "country",
            "language",
            "alpha2_code",
            "language_code",
            "date_format_enum",
            "hour_format",
            "temp_format",
            "is_active",
            "date_separator",
            "zwave_region_frequency_code",
            "zwave_region_frequency",
            "zwave_region_prop_values",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,country,language,alpha2_code,
                             language_code,date_format_enum,hour_format,temp_format,is_active,date_separator,
                             zwave_region_frequency_code,zwave_region_frequency,zwave_region_prop_values)
                              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("country", ""),
            data.get("language", ""),
            data.get("alpha2_code", ""),
            data.get("language_code", ""),
            data.get("date_format_enum", ""),
            data.get("hour_format", ""),
            data.get("temp_format", ""),
            data.get("is_active", ""),
            data.get("date_separator", ""),
            data.get("zwave_region_frequency_code", ""),
            data.get("zwave_region_frequency", ""),
            data.get("zwave_region_prop_values", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableDashboardMsgs(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.DashboardMessagesContentProvider/dashboard_msgs"
        self._table = "dashboard_msgs"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "msg_id",
            "title",
            "description",
            "received_time",
            "start_time",
            "end_time",
            "read",
            "mime_type",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,msg_id,title,description,
                             received_time,start_time,end_time,read,mime_type) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version"),
            data.get("opr"),
            data.get("partition_id"),
            data.get("msg_id"),
            data.get("title"),
            data.get("description"),
            data.get("received_time"),
            data.get("start_time"),
            data.get("end_time"),
            data.get("read"),
            data.get("mime_type")))

        self._db.commit()



#!/usr/bin/env python3

import argparse
import asyncio
import logging
import os
import sys

from qolsys_controller import qolsys_controller
from qolsys_controller.errors import QolsysMqttError, QolsysSqlError, QolsysSslError


async def main() -> None:  # noqa: D103

    cli_parser = argparse.ArgumentParser()
    cli_parser.add_argument("--panel-ip", help="Qolsys Panel IP", default="")
    cli_parser.add_argument("--plugin-ip", help="Plugin IP", default="")
    cli_parser.add_argument("--config-dir", help="Configuration Directory", default="./config/")
    cli_parser.add_argument("--pki-autodiscovery", help="Enable PKI Autodiscovery", action="store_true")
    cli_parser.add_argument("--debug", help="Verbose MQTT Traffic", action="store_true")

    args = cli_parser.parse_args()

    logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(module)s: %(message)s")
    LOGGER = logging.getLogger(__name__)

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

    Panel = qolsys_controller()

    # Select plugin
    Panel.select_plugin("remote")
    Panel.settings.config_directory = args.config_dir
    Panel.plugin.settings.panel_ip = args.panel_ip
    Panel.plugin.settings.plugin_ip = args.plugin_ip
    Panel.plugin.settings.random_mac = ""

    # Additionnal remote plugin config
    Panel.plugin.check_user_code_on_disarm = False
    Panel.plugin.log_mqtt_mesages = args.debug
    Panel.plugin.auto_discover_pki = args.pki_autodiscovery

    # Configure remote plugin
    if not await Panel.plugin.config(start_pairing=True):
        LOGGER.debug("Error Configuring remote plugin")
        return

    try:
        await Panel.plugin.start_operation()

    except QolsysMqttError:
        LOGGER.debug("QolsysMqttError")

    except QolsysSslError:
        LOGGER.debug("QolsysSslError")

    except QolsysSqlError:
        LOGGER.debug("QolsysSqlError")

    if not Panel.plugin.connected:
        LOGGER.error("Panel not ready for operation")
        return

    LOGGER.debug("Qolsys Panel Ready for operation")

    stop_event = asyncio.Event()
    await stop_event.wait()

# Change to the "Selector" event loop if platform is Windows
if sys.platform.lower() == "win32" or os.name.lower() == "nt":
    from asyncio import WindowsSelectorEventLoopPolicy, set_event_loop_policy
    set_event_loop_policy(WindowsSelectorEventLoopPolicy())

asyncio.run(main())



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableUser(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.UserContentProvider/user"
        self._table = "user"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "username",
            "userPin",
            "expirydate",
            "usertype",
            "userid",
            "lastname",
            "check_in",
            "hash_user",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,username,userPin,
                             expirydate,usertype,userid,lastname,check_in,hash_user) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("username", ""),
            data.get("userPin", ""),
            data.get("expirydate", ""),
            data.get("usertype", ""),
            data.get("userid", ""),
            data.get("lastname", ""),
            data.get("check_in", ""),
            data.get("hash_user", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableZwaveNode(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.ZwaveContentProvider/zwave_node"
        self._table = "zwave_node"
        self._abort_on_error = True

        self._columns = [
            "_id",
            "version",
            "opr TEXT",
            "partition_id",
            "node_id",
            "node_name",
            "node_type",
            "node_status",
            "node_secure_cmd_cls",
            "node_battery_level",
            "node_battery_level_value",
            "is_node_listening_node",
            "basic_report_value",
            "switch_multilevel_report_value",
            "basic_device_type",
            "generic_device_type",
            "specific_device_type",
            "num_secure_command_class",
            "secure_command_class",
            "manufacture_id",
            "product_type",
            "product_id",
            "library_type_version",
            "protocol_version",
            "protocol_sub_version",
            "application_version",
            "application_sub_version",
            "capability",
            "command_class_list",
            "lenof_command_class_list",
            "security",
            "library_type",
            "last_updated_date",
            "node_battery_level_updated_time",
            "basic_report_updated_time",
            "switch_multilevel_report_updated_time",
            "multi_channel_details",
            "rediscover_status",
            "last_rediscover_time",
            "neighbour_info",
            "last_node_test_time",
            "endpoint",
            "endpoint_details",
            "device_wakeup_time",
            "role_type",
            "is_device_sleeping",
            "counters_passed",
            "counters_failed",
            "group_id",
            "command_classes_version",
            "paired_status",
            "device_dsk",
            "endpoint_secure_cmd_cls",
            "s2_security_keys",
            "device_protocol",
            "is_device_hidden",
            "ime_data",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(
            f"""INSERT INTO {self.table} (_id,version,opr,partition_id,node_id,node_name,node_type,node_status,
            node_secure_cmd_cls,node_battery_level,node_battery_level_value,is_node_listening_node,basic_report_value,
            switch_multilevel_report_value,basic_device_type,generic_device_type,specific_device_type,num_secure_command_class,
            secure_command_class,manufacture_id,product_type,product_id,library_type_version,protocol_version,protocol_sub_version,
            application_version,application_sub_version,capability,command_class_list,lenof_command_class_list,security,library_type,
            last_updated_date,node_battery_level_updated_time,basic_report_updated_time,switch_multilevel_report_updated_time,
            multi_channel_details,rediscover_status,last_rediscover_time,neighbour_info,last_node_test_time,endpoint,endpoint_details,
            device_wakeup_time,role_type,is_device_sleeping,counters_passed,counters_failed,group_id,command_classes_version,
            paired_status,device_dsk,endpoint_secure_cmd_cls,s2_security_keys,device_protocol,is_device_hidden,ime_data)
              VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,
            ?,?,?,?,?,?,?,?)""",
            (
                            data.get("_id"),
                            data.get("version", ""),
                            data.get("opr", ""),
                            data.get("partition_id", ""),
                            data.get("node_id", ""),
                            data.get("node_name", ""),
                            data.get("node_type", ""),
                            data.get("node_status", ""),
                            data.get("node_secure_cmd_cls", ""),
                            data.get("node_battery_level", ""),
                            data.get("node_battery_level_value", ""),
                            data.get("is_node_listening_node", ""),
                            data.get("basic_report_value", ""),
                            data.get("switch_multilevel_report_value", ""),
                            data.get("basic_device_type", ""),
                            data.get("generic_device_type", ""),
                            data.get("specific_device_type", ""),
                            data.get("num_secure_command_class", ""),
                            data.get("secure_command_class", ""),
                            data.get("manufacture_id", ""),
                            data.get("product_type", ""),
                            data.get("product_id", ""),
                            data.get("library_type_version", ""),
                            data.get("protocol_version", ""),
                            data.get("protocol_sub_version", ""),
                            data.get("application_version", ""),
                            data.get("application_sub_version", ""),
                            data.get("capability", ""),
                            data.get("command_class_list", ""),
                            data.get("lenof_command_class_list", ""),
                            data.get("security", ""),
                            data.get("library_type", ""),
                            data.get("last_updated_date", ""),
                            data.get("node_battery_level_updated_time", ""),
                            data.get("basic_report_updated_time", ""),
                            data.get("switch_multilevel_report_updated_time", ""),
                            data.get("multi_channel_details", ""),
                            data.get("rediscover_status", ""),
                            data.get("last_rediscover_time", ""),
                            data.get("neighbour_info", ""),
                            data.get("last_node_test_time", ""),
                            data.get("endpoint", ""),
                            data.get("endpoint_details", ""),
                            data.get("device_wakeup_time", ""),
                            data.get("role_type", ""),
                            data.get("is_device_sleeping", ""),
                            data.get("counters_passed", ""),
                            data.get("counters_failed", ""),
                            data.get("group_id", ""),
                            data.get("command_classes_version", ""),
                            data.get("paired_status", ""),
                            data.get("device_dsk", ""),
                            data.get("endpoint_secure_cmd_cls", ""),
                            data.get("s2_security_keys", ""),
                            data.get("device_protocol", ""),
                            data.get("is_device_hidden", ""),
                            data.get("ime_data", "")))
        self._db.commit()



import logging

from qolsys_controller.zwave_device import QolsysZWaveDevice

LOGGER = logging.getLogger(__name__)


class QolsysGeneric(QolsysZWaveDevice):

    def __init__(self, zwave_dict: dict) -> None:
        super().__init__(zwave_dict)



#!/usr/bin/env python3

import asyncio
import logging
import os
import sys

from qolsys_controller.controller import QolsysController
from qolsys_controller.errors import QolsysMqttError, QolsysSqlError, QolsysSslError

logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(module)s: %(message)s")
LOGGER = logging.getLogger(__name__)


async def main() -> None:  # noqa: D103

    remote = QolsysController()

    # Select plugin
    remote.select_plugin("remote")

    remote.plugin.settings.config_directory = "./config/"
    remote.plugin.settings.panel_ip = "192.168.10.220"
    remote.plugin.settings.plugin_ip = "192.168.10.73"
    remote.plugin.settings.random_mac = ""  # Example: F2:16:3E:33:ED:20

    # Additionnal remote plugin config
    remote.plugin.check_user_code_on_disarm = False  # Check user code in user.conf file
    remote.plugin.log_mqtt_mesages = False  # Enable for MQTT debug purposes
    remote.plugin.auto_discover_pki = True

    # Configure remote plugin
    if not await remote.plugin.config(start_pairing=True):
        LOGGER.debug("Error Configuring remote plugin")
        return

    try:
        await remote.plugin.start_operation()

    except QolsysMqttError:
        LOGGER.debug("QolsysMqttError")

    except QolsysSslError:
        LOGGER.debug("QolsysSslError")

    except QolsysSqlError:
        LOGGER.debug("QolsysSqlError")

    if not remote.plugin.connected:
        LOGGER.error("Panel not ready for operation")
        return

    LOGGER.debug("Qolsys Panel Ready for operation")

    # Change Z-Wave dimmer
    # node_id: z-wane device id
    # level: 0-99, -1 to switch from off to previous on dimmer level
    # await asyncio.sleep(3)
    # await remote.plugin.command_zwave_switch_multi_level(node_id=6,level=99)

    # DISARM
    # await asyncio.sleep(3)
    # await remote.plugin.command_disarm(partition_id="0",
    #                                   user_code="1111")

    # ARM_STAY
    await asyncio.sleep(3)
    await remote.plugin.command_arm(partition_id="0",
                                    arming_type="ARM-NIGHT",
                                    user_code="1111",
                                    exit_sounds=False,
                                    instant_arm=False)

    # DISARM
    await asyncio.sleep(3)
    await remote.plugin.command_disarm(partition_id="0",
                                       user_code="1111")

    # ARM_AWAY
    # await asyncio.sleep(3)
    # await remote.plugin.command_arm(partition_id=0,
    #                               arming_type='ARM-AWAY',
    #                               user_code='1111',
    #                                exit_sounds=True,
    #                               instant_arm=False)

    # Use an asyncio.Event to keep the program running efficiently
    stop_event = asyncio.Event()
    await stop_event.wait()

# Change to the "Selector" event loop if platform is Windows
if sys.platform.lower() == "win32" or os.name.lower() == "nt":
    from asyncio import WindowsSelectorEventLoopPolicy, set_event_loop_policy
    set_event_loop_policy(WindowsSelectorEventLoopPolicy())

asyncio.run(main())



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableHistory(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.HistoryContentProvider/history"
        self._table = "history"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "device",
            "events",
            "time",
            "ack",
            "type",
            "feature1",
            "device_id",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,device,events,time,ack,type,
                             feature1,device_id) VALUES (?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("device", ""),
            data.get("events", ""),
            data.get("time", ""),
            data.get("ack", ""),
            data.get("type", ""),
            data.get("feature1", ""),
            data.get("device_id", "")))

        self._db.commit()



from enum import StrEnum


class PartitionSystemStatus(StrEnum):
    ARM_STAY = "ARM-STAY"
    ARM_AWAY = "ARM-AWAY"
    ARM_NIGHT = "ARM-NIGHT"
    DISARM = "DISARM"
    ARM_AWAY_EXIT_DELAY = "ARM-AWAY-EXIT-DELAY"
    ARM_STAY_EXIT_DELAY = "ARM-STAY-EXIT-DELAY"
    ARM_NIGHT_EXIT_DELAY = "ARM-NIGHT-EXIT-DELAY"
    UNKNOWN = "UNKNOWN"


class PartitionAlarmState(StrEnum):
    NONE = "None"
    DELAY = "Delay"
    ALARM = "Alarm"
    UNKNOWN = "UNKNOWN"


class PartitionAlarmType(StrEnum):
    POLICE_EMERGENCY = "Police Emergency"
    FIRE_EMERGENCY = "Fire Emergency"
    AUXILIARY_EMERGENCY = "Auxiliary Emergency"
    SILENT_AUXILIARY_EMERGENCY = "Silent Auxiliary Emergency"
    SILENT_POLICE_EMERGENCY = "Silent Police Emergency"


class ZoneStatus(StrEnum):
    OPEN = "Open"
    CLOSED = "Closed"
    ACTIVE = "Active"
    INACTIVE = "Inactive"
    ACTIVATED = "Activated"
    IDLE = "Idle"
    UNREACHABLE = "Unreachable"
    TAMPERED = "Tampered"
    SYNCHONIZING = "Synchonizing"
    CONNECTED = "connected"
    FAILURE = "Failure"


class ZoneSensorType(StrEnum):
    DOOR_WINDOW = "Door_Window"
    MOTION = "Motion"
    GLASS_BREAK = "GlassBreak"
    KEY_FOB = "KeyFob"
    KEYPAD = "Keypad"
    AUXILIARY_PENDANT = "Auxiliary Pendant"
    SMOKE_DETECTOR = "SmokeDetector"
    CO_DETECTOR = "CODetector"
    # HARDWIRE_TRANSLATOR = "" # TBD
    # WIRELESS_TRANSLATOR = "" #TBD
    TEMPERATURE = "Temperature"
    HEAT = "Heat"
    WATER = "Water"
    SHOCK = "Shock"
    FREEZE = "Freeze"
    TILT = "Tilt"
    DOORBELL = "Doorbell"
    SMOKE_M = "Smoke_M"
    # DOOR_WINDOW_M = "" #TBD
    # OCCUPANCY = ""  #TBD
    SIREN = "Siren"
    # HIGH_TEMPERATURE = "" # TBD
    PANEL_MOTION = "Panel Motion"
    PANEL_GLASS_BREAK = "Panel Glass Break"
    BLUETOOTH = "Bluetooth"
    TAKEOVER_MODULE = "TakeoverModule"
    TRANSLATOR = "Translator"


class ZoneSensorGroup(StrEnum):
    FIXED_INTRUSION = "fixedintrusion"
    MOBILE_INTRUSION = "mobileintrusion"
    FIXED_SILENT = "fixedsilentkey"
    MOBILE_SILENT = "mobilesilentkey"
    FIXED_AUXILIARY = "fixedmedical"
    FIXED_SILENT_AUXILIARY = "fixedsilentmedical"
    MOBILE_AUXILIARY = "mobilemedical"
    MOBILE_SILENT_AUXILIARY = "mobilesilentmedical"
    SAFETY_MOTION = "safetymotion"
    # ENTRY_EXIT_NORMAL_DELAY = "" #TBD
    # ENTRY_EXIT_LONG_DELAY = "" #TBD
    # INSTANT_PERIMETER_DW = "" #TBD
    # INSTRANT_INTERIOR_DOOR = "" #TBD
    # AWAY_INSTANT_FOLLOWER_DELAY = "" #TBD
    # LOCAL_SAFETY_SENSOR = "" #TBD
    # REPORTING_SAFETY_SENSOR = "" #TBD
    # DELAYED_REPORTING_SAFETY_SENSOR = "" #TBD
    # AWAY_INSTANT_MOTION = "" #TBD
    # STAY_INSTANT_MOTION = "" #TBD
    # STAY_DELAY_MOTION = "" #TBD
    # AWAY_DELAY_MOTION = "" #TBD
    # GLASS_BREAK = "" # TBD
    # GLASS_BREAK_AWAY_ONLY = "" # TBD
    # SMOKE_HEAT = "" # TBD
    # CO = "" # TBD
    # TAKEOVER = "" #TBD
    # GARAGE_TILT_SAFETY = "" # TBD
    # WATER_SENSOR = "" # TBD
    # WATER_SENSOR_NON_REPORTING = "" # TBD
    # SHOCK_GLASS_BREAK = "" # TBD
    # SHOCK_GLASS_BREAK_AWAY_ONLY = "" # TBD
    # FREEZE = "" # TBD
    # FREEZE_NON_REPORTING = "" # TBD
    # TEMP_REPORTING = "" #TBD
    # TEMP_NON_REPORTING = "" #TBD
    # SIREN = "" # TBD


class ZWaveNodeStatus(StrEnum):
    NORMAL = "Normal"
    UNREACHABLE = "Unreachable"



import logging
from pathlib import Path

LOGGER = logging.getLogger(__name__)


class QolsysSettings:

    def __init__(self) -> None:

        # Plugin
        self._plugin_ip = ""
        self._random_mac = ""
        self._panel_mac = ""
        self._panel_ip = ""

        self._config_directory: Path = Path()
        self._pki_directory: Path = Path()
        self._media_directory: Path = Path()
        self._users_file_path: Path = Path()

        # Pki
        self._key_size: int = 2048

        # MQTT
        self._mqtt_timeout: int = 30
        self._mqtt_ping: int = 600
        self._mqtt_remote_client_id = ""

    @property
    def random_mac(self) -> str:
        return self._random_mac

    @random_mac.setter
    def random_mac(self, random_mac: str) -> None:
        self._random_mac = random_mac.lower()

    @property
    def plugin_ip(self) -> str:
        return self._plugin_ip

    @property
    def panel_mac(self) -> str:
        return self._panel_mac

    @panel_mac.setter
    def panel_mac(self, panel_mac: str) -> None:
        self._panel_mac = panel_mac

    @property
    def panel_ip(self) -> str:
        return self._panel_ip

    @panel_ip.setter
    def panel_ip(self, panel_ip: str) -> None:
        self._panel_ip = panel_ip

    @plugin_ip.setter
    def plugin_ip(self, plugin_ip: str) -> None:
        self._plugin_ip = plugin_ip

    @property
    def config_directory(self) -> str:
        return self._config_directory

    @config_directory.setter
    def config_directory(self, config_directory: str) -> None:
        self._config_directory = Path(config_directory)
        self._pki_directory = self._config_directory.joinpath("pki")
        self._media_directory = self._config_directory.joinpath("media")
        self._users_file_path = self._config_directory.joinpath("users.conf")

    @property
    def pki_directory(self) -> Path:
        return self._pki_directory

    @property
    def users_file_path(self) -> Path:
        return self._users_file_path

    @property
    def key_size(self) -> int:
        return self._key_size

    @property
    def mqtt_timeout(self) -> int:
        return self._mqtt_timeout

    @property
    def mqtt_ping(self) -> int:
        return self._mqtt_ping

    @mqtt_ping.setter
    def mqtt_ping(self, ping:int) -> None:
        self._mqtt_ping = ping

    @property
    def mqtt_remote_client_id(self) -> str:
        return self._mqtt_remote_client_id

    @mqtt_remote_client_id.setter
    def mqtt_remote_client_id(self,client_id:str) -> None:
        self._mqtt_remote_client_id = client_id

    def check_panel_ip(self) -> bool:
        if self._panel_ip == "":
            LOGGER.debug("Invalid Panel IP:  %s", self._panel_ip)
            return False

        LOGGER.debug("Found Panel IP: %s", self._panel_ip)
        return True

    def check_plugin_ip(self) -> bool:
        if self._plugin_ip == "":
            LOGGER.debug("Invalid Plugin IP:  %s", self._plugin_ip)
            return False

        LOGGER.debug("Found Plugin IP: %s", self._plugin_ip)
        return True

    def check_config_directory(self, create: bool = True) -> bool:
        if not self.config_directory.is_dir():
            if not create:
                LOGGER.debug("config_directory not found:  %s", self.config_directory)
                return False

            # Create config directory if not found
            LOGGER.debug("Creating config_directory: %s", self.config_directory)
            try:
                self.config_directory.mkdir(parents=True)
            except PermissionError:
                LOGGER.exception("Permission denied: Unable to create: %s", self.config_directory)
                return False
            except Exception:
                LOGGER.exception("Error creating config_directory: %s", self.config_directory)
                return False

        LOGGER.debug("Using config_directory: %s", self.config_directory.resolve())

        # Create pki directory if not found
        if not self.pki_directory.is_dir():
            LOGGER.debug("Creating pki_directory: %s", self.pki_directory.resolve())
            try:
                self.pki_directory.mkdir(parents=True)
            except PermissionError:
                LOGGER.exception("Permission denied: Unable to create: %s", self.pki_directory.resolve())
                return False
            except Exception:
                LOGGER.exception("Error creating pki_directory: %s", self.pki_directory.resolve())
                return False

        LOGGER.debug("Using pki_directory: %s", self.pki_directory.resolve())

        # Create media directory if not found
        if not self._media_directory.is_dir():
            LOGGER.debug("Creating media_directory: %s", self._media_directory.resolve())
            try:
                self._media_directory.mkdir(parents=True)
            except PermissionError:
                LOGGER.exception("Permission denied: Unable to create: %s", self._media_directory.resolve())
                return False
            except Exception:
                LOGGER.exception("Error creating media_directory: %s", self._media_directory.resolve())
                return False

        LOGGER.debug("Using media_directory: %s", self._media_directory.resolve())

        return True



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableZwaveAssociationGroup(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.ZwaveAssociationGroupContentProvider/zwave_association_group"
        self._table = "zwave_association_group"
        self._abort_on_error = False

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "group_name",
            "associated_nodes",
            "group_id",
            "created_date",
            "last_updated_date",
            "group_command_class",
            "max_supported_nodes",
            "node_id",
            "endpoint",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,group_name,associated_nodes,
                             group_id,created_date,last_updated_date,group_command_class,max_supported_nodes,node_id,
                             endpoint) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("group_name", ""),
            data.get("associated_nodes", ""),
            data.get("group_id", ""),
            data.get("created_date", ""),
            data.get("last_updated_date", ""),
            data.get("group_command_class", ""),
            data.get("max_supported_nodes", ""),
            data.get("node_id", ""),
            data.get("endpoint", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from .table import QolsysTable

LOGGER = logging.getLogger(__name__)


class QolsysTableDoorLock(QolsysTable):

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        super().__init__(db, cursor)
        self._uri = "content://com.qolsys.qolsysprovider.DoorLocksContentProvider/doorlock"
        self._table = "doorlock"
        self._abort_on_error = True

        self._columns = [
            "_id",
            "version",
            "opr",
            "partition_id",
            "doorlock_name",
            "status",
            "node_id",
            "created_by",
            "created_date",
            "updated_by",
            "last_updated_date",
            "remote_arming",
            "keyfob_arming",
            "panel_arming",
            "endpoint",
            "paired_status",
        ]

        self._create_table()

    def insert(self, data: dict) -> None:
        self._cursor.execute(f"""INSERT INTO {self.table} (_id,version,opr,partition_id,doorlock_name,status,node_id,
                             remote_arming,keyfob_arming,panel_arming,created_by,created_date,updated_by,last_updated_date,
                             endpoint,paired_status) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)""", (
            data.get("_id"),
            data.get("version", ""),
            data.get("opr", ""),
            data.get("partition_id", ""),
            data.get("doorlock_name", ""),
            data.get("status", ""),
            data.get("node_id", ""),
            data.get("remote_arming", ""),
            data.get("keyfob_arming", ""),
            data.get("panel_arming", ""),
            data.get("created_by", ""),
            data.get("created_date", ""),
            data.get("updated_by", ""),
            data.get("last_updated_date", ""),
            data.get("endpoint", ""),
            data.get("paired_status", "")))

        self._db.commit()



import logging  # noqa: INP001
import sqlite3

from qolsys_controller.errors import QolsysSqlError

LOGGER = logging.getLogger(__name__)


class QolsysTable:

    def __init__(self, db: sqlite3.Connection, cursor: sqlite3.Cursor) -> None:
        self._db: sqlite3.Connection = db
        self._cursor: sqlite3.Cursor = cursor
        self._uri: str = ""
        self._table: str = ""
        self._columns: list[str] = []
        self._abort_on_error: bool = False

    @property
    def uri(self) -> str:
        return self._uri

    @property
    def table(self) -> str:
        return self._table

    def _create_table(self) -> None:
        if not self._columns:
            msg = "The column list must not be empty."
            raise ValueError(msg)

        primary_key = self._columns[0]
        other_columns = self._columns[1:]

        column_defs = [f"{primary_key} TEXT PRIMARY KEY"]
        column_defs += [f"{col} TEXT" for col in other_columns]

        try:
            query = f"CREATE TABLE {self._table} ({', '.join(column_defs)})"
            self._cursor.execute(query)
            self._db.commit()

        except sqlite3.Error as err:
            error = QolsysSqlError({
                "table": self.table,
                "query": query,
                "columns": self._columns,
            })

            if self._abort_on_error:
                raise error from err

    def clear(self) -> None:
        try:
            query = f"DELETE from {self.table}"
            self._cursor.execute(query)
            self._db.commit()

        except sqlite3.Error as err:
            error = QolsysSqlError({
                "table": self.table,
                "query": query,
                "columns": self._columns,
            })

            if self._abort_on_error:
                raise error from err

    def update(self, selection: str, selection_argument: str, content_value: str) -> None:
        # selection: 'zone_id=?, parition_id=?'

        # Firmware 4.4.1 is sending contentValues as string
        # selection_argument: '[3,1]'
        # Firmware 4.6.1: 
        # selection_argument: ['cc:4b:73:86:5c:89']

        #  "contentValues":{"partition_id":"0","sensorgroup":"safetymotion","sensorstatus":"Idle"}"

        # Panel is sending query parameter for db update in text string
        # Have not found a way to make it work with parametrized query yet
        # Using f string concat for moment ...

        # New Values to update in table
        # To change for parametrized
        db_value = ",".join([f"{key}='{value}'" for key, value in content_value.items()])

        # Selection Argument
        # Panel send selection_argument as list in Firmware 4.6.1
        if(type(selection_argument) is not list):
            #Firmware 4.4.1, seletion_argument is sent as a string
            selection_argument = selection_argument.strip("[]")
            selection_argument = [item.strip() for item in selection_argument.split(",")]

        try:
            query = f"UPDATE {self.table} SET {db_value} WHERE {selection}"
            self._cursor.execute(query, selection_argument)
            self._db.commit()

        except sqlite3.Error as err:
            error = QolsysSqlError({
                "table": self.table,
                "query": query,
                "columns": self._columns,
                "content_value": content_value,
                "selection": selection,
                "selection_argument": selection_argument,
            })

            if self._abort_on_error:
                raise error from err

    def insert(self) -> None:
        pass

    def delete(self, selection: str, selection_argument: str) -> None:
        # selection: 'zone_id=?, parition_id=?'
        # selection_argument: '[3,1]'

        # Selection Argument
        # Panel send selection_argument as list in Firmware 4.6.1
        if(type(selection_argument) is not list):
            #Firmware 4.4.1, seletion_argument is sent as a string
            selection_argument = selection_argument.strip("[]")
            selection_argument = [item.strip() for item in selection_argument.split(",")]

        try:
            query = f"DELETE FROM {self.table} WHERE {selection}"
            self._cursor.execute(query, selection_argument)
            self._db.commit()

        except sqlite3.Error as err:
            error = QolsysSqlError({
                "table": self.table,
                "query": query,
                "columns": self._columns,
                "selection": selection,
                "selection_argument": selection_argument,
            })

            if self._abort_on_error:
                raise error from err
